@@ -48,8 +48,11 @@ import firefly.infrastructure as ffi
 import inflection
 import yaml
 from botocore.exceptions import ClientError
+from troposphere.dynamodb import Table, AttributeDefinition, KeySchema
+from troposphere.events import Target, Rule
+
 from firefly_aws import S3Service, ResourceNameAware
-from troposphere import Template, GetAtt, Ref, Parameter, Output, Export, ImportValue, Join
+from troposphere import Template, GetAtt, Ref, Parameter, Output, Export, ImportValue, Join, Equals
 from troposphere.apigatewayv2 import Api, Stage, Deployment, Integration, Route
 from troposphere.awslambda import Function, Code, VPCConfig, Environment, Permission, EventSourceMapping
 from troposphere.cloudwatch import Alarm, MetricDimension
@@ -191,6 +194,37 @@ class AwsAgent(ff.ApplicationService, ResourceNameAware):
             **params
         ))
 
+        # Timers
+        for cls, _ in context.command_handlers.items():
+            if cls.has_timer():
+                timer = cls.get_timer()
+                if timer.environment is not None and timer.environment != self._env:
+                    continue
+                if isinstance(timer.command, str):
+                    timer_name = timer.command
+                else:
+                    timer_name = timer.command.__name__
+
+                target = Target(
+                    f'{self._service_name(service.name)}AsyncTarget',
+                    Arn=GetAtt(f'{self._lambda_resource_name(service.name)}Async', 'Arn'),
+                    Id=f'{self._lambda_resource_name(service.name)}Async',
+                    Input=f'{{"_context": "{context.name}", "_type": "command", "_name": "{cls.__name__}"}}'
+                )
+                rule = template.add_resource(Rule(
+                    f'{timer_name}TimerRule',
+                    ScheduleExpression=f'cron({timer.cron})',
+                    State='ENABLED',
+                    Targets=[target]
+                ))
+                template.add_resource(Permission(
+                    f'{timer_name}TimerPermission',
+                    Action='lambda:invokeFunction',
+                    Principal='events.amazonaws.com',
+                    FunctionName=Ref(async_lambda),
+                    SourceArn=GetAtt(rule, 'Arn')
+                ))
+
         integration = template.add_resource(Integration(
             self._integration_name(context.name),
             ApiId=ImportValue(self._rest_api_reference()),
@@ -319,6 +353,28 @@ class AwsAgent(ff.ApplicationService, ResourceNameAware):
                     DependsOn=[queue, dlq]
                 ))
 
+        # DynamoDB Table
+
+        ddb_table = template.add_resource(Table(
+            self._ddb_resource_name(context.name),
+            TableName=self._ddb_table_name(context.name),
+            AttributeDefinitions=[
+                AttributeDefinition(AttributeName='pk', AttributeType='S'),
+                AttributeDefinition(AttributeName='sk', AttributeType='S'),
+            ],
+            BillingMode='PAY_PER_REQUEST',
+            KeySchema=[
+                KeySchema(AttributeName='pk', KeyType='HASH'),
+                KeySchema(AttributeName='sk', KeyType='RANGE'),
+            ]
+        ))
+
+        template.add_output(Output(
+            "DDBTable",
+            Value=Ref(ddb_table),
+            Description="Document table"
+        ))
+
         self.info('Deploying stack')
         stack_name = self._stack_name(context.name)
         try:
@@ -330,17 +386,17 @@ class AwsAgent(ff.ApplicationService, ResourceNameAware):
             else:
                 raise e
 
-        self._execute_ddl(context)
+        self._migrate_schema(context)
 
         self.info('Done')
 
-    def _execute_ddl(self, context: ff.Context):
+    def _migrate_schema(self, context: ff.Context):
         for entity in context.entities:
             if issubclass(entity, ff.AggregateRoot) and entity is not ff.AggregateRoot:
                 try:
                     repository = self._registry(entity)
                     if isinstance(repository, ffi.RdbRepository):
-                        repository.execute_ddl()
+                        repository.migrate_schema()
                 except ff.FrameworkError:
                     self.debug('Could not execute ddl for entity %s', entity)
 
@@ -398,9 +454,11 @@ class AwsAgent(ff.ApplicationService, ResourceNameAware):
         with open('firefly.yml', 'w') as fp:
             fp.write(yaml.dump(self._configuration.all))
 
-        subprocess.call(['find', '.', '-name', '"*.so"', '|', 'xargs', 'strip'])
-        subprocess.call(['find', '.', '-name', '"*.so.*"', '|', 'xargs', 'strip'])
-        subprocess.call(['find', '.', '-name', '"*.pyc"', '-delete'])
+        subprocess.call(['find', '.', '-name', '"*.so"', '-exec', 'strip', '{}', ';'])
+        subprocess.call(['find', '.', '-name', '"*.so.*"', '-exec', 'strip', '{}', ';'])
+        subprocess.call(['find', '.', '-name', '"*.pyc"', '-exec', 'rm', '-Rf', '{}', ';'])
+        subprocess.call(['find', '.', '-name', 'tests', '-type', 'd', '-exec', 'rm', '-R', '{}', ';'])
+
         file_name = self._code_key.split('/')[-1]
         subprocess.call(['zip', '-r', f'../{file_name}', '.'])
         os.chdir('..')
@@ -448,8 +506,7 @@ class AwsAgent(ff.ApplicationService, ResourceNameAware):
         memory_size = template.add_parameter(Parameter(
             f'{self._stack_name()}MemorySize',
             Type=NUMBER,
-            Default='3008',
-
+            Default='3008'
         ))
 
         timeout_gateway = template.add_parameter(Parameter(
@@ -590,6 +647,7 @@ class AwsAgent(ff.ApplicationService, ResourceNameAware):
                                     'secretsmanager:GetSecretValue',
                                     'sns:*',
                                     'sqs:*',
+                                    'dynamodb:*',
                                 ],
                                 'Resource': '*',
                                 'Effect': 'Allow',
@@ -648,6 +706,7 @@ class AwsAgent(ff.ApplicationService, ResourceNameAware):
             'CONTEXT': context.name,
             'REGION': self._region,
             'BUCKET': self._bucket,
+            'DDB_TABLE': self._ddb_table_name(context.name),
         }
         if env is not None:
             defaults.update(env)
